package com.iconplus.smartproc.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.iconplus.smartproc.exception.ResourceNotFoundException;
import com.iconplus.smartproc.model.entity.SumberDana;
import com.iconplus.smartproc.repository.SumberDanaRepository;

@CrossOrigin(origins = "http://localhost:8080", allowCredentials = "true")
@RestController
@RequestMapping("/api/sumberdana")
public class SumberDanaController {
        @Autowired
        private SumberDanaRepository sumberdanaRepository;

        //get all data
        @GetMapping
        public List<SumberDana> getAllSumberdanas(){
            return sumberdanaRepository.findAll();
        }

        // create
        @PostMapping
        public SumberDana createSumberdana(@RequestBody SumberDana sumberdana) {
            return sumberdanaRepository.save(sumberdana);
        }

        // get sumberdana by id rest api
        @GetMapping("/{id}")
        public ResponseEntity<SumberDana> getSumberdanaById(@PathVariable Long id) {
            SumberDana sumberdana = sumberdanaRepository.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("Sumberdana not exist with id :" + id));
            return ResponseEntity.ok(sumberdana);
        }

        // update sumberdana rest api
        @PutMapping("/{id}")
        public ResponseEntity<SumberDana> updateSumberdana(@PathVariable Long id, @RequestBody SumberDana sumberDanaDetails){
            SumberDana sumberdana = sumberdanaRepository.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("Sumberdana not exist with id :" + id));

            sumberdana.setSumberDana(sumberDanaDetails.getSumberDana());
            sumberdana.setKeterangan(sumberDanaDetails.getKeterangan());

            SumberDana updatedSumberDana =  sumberdanaRepository.save(sumberdana);
            return ResponseEntity.ok(updatedSumberDana);
        }

        // delete sumberdana rest api
        @DeleteMapping("/{id}")
        public ResponseEntity<Map<String, Boolean>> deleteSumberdana(@PathVariable Long id){
            SumberDana sumberdana = sumberdanaRepository.findById(id)
                    .orElseThrow(() -> new ResourceNotFoundException("Sumberdana not exist with id :" + id));

            sumberdanaRepository.delete(sumberdana);
            Map<String, Boolean> response = new HashMap<>();
            response.put("deleted", Boolean.TRUE);
            return ResponseEntity.ok(response);
        }




}
